@@ -1,24 +1,7 @@
-
-/*
- * x509.c
- *
- * Copyright (C) AB Strakt 2001, All rights reserved
- *
- * Certificate (X.509) handling code, mostly thin wrappers around OpenSSL.
- * See the file RATIONALE for a short explanation of why this module was written.
- *
- * Reviewed 2001-07-23
- */
 #include <Python.h>
-#include <datetime.h>
 #define crypto_MODULE
 #include "crypto.h"
 
-
-
-static char *CVSid =
-    "@(#) $Id: x509.c,v 1.9 2008/10/20 17:58:51 acasajus Exp $";
-
 /*
  * X.509 is a standard for digital certificates.  See e.g. the OpenSSL homepage
  * http://www.openssl.org/ for more information
@@ -60,8 +43,7 @@ crypto_X509_set_version( crypto_X509Obj * self, PyObject * args )
 
     X509_set_version( self->x509, version );
 
-    Py_INCREF( Py_None );
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 static char crypto_X509_get_serial_number_doc[] = "\n\
@@ -78,7 +60,7 @@ crypto_X509_get_serial_number( crypto_X509Obj * self, PyObject * args )
    ASN1_INTEGER *asn1_i;
    int length;
    PyObject *pySerial;
-   char *cbuf;
+   unsigned char *cbuf;
 
    if ( !PyArg_ParseTuple( args, ":get_serial_number" ) )
       return NULL;
@@ -89,7 +71,7 @@ crypto_X509_get_serial_number( crypto_X509Obj * self, PyObject * args )
    if( !length )
       return PyString_FromString( "" );
    pySerial = PyString_FromStringAndSize( NULL, length );
-   cbuf = PyString_AsString( pySerial );
+   cbuf = (unsigned char*)PyString_AsString( pySerial );
    length = i2c_ASN1_INTEGER( asn1_i, &cbuf);
 
    return pySerial;
@@ -107,7 +89,7 @@ Returns:   None\n\
 static PyObject *
 crypto_X509_set_serial_number( crypto_X509Obj * self, PyObject * args )
 {
-   char *serial;
+   const  unsigned char *serial;
    long serialLength;
    ASN1_INTEGER *asn1_i;
 
@@ -117,8 +99,7 @@ crypto_X509_set_serial_number( crypto_X509Obj * self, PyObject * args )
    asn1_i = X509_get_serialNumber( self->x509 );
    c2i_ASN1_INTEGER( &asn1_i, &serial, serialLength );
 
-   Py_INCREF( Py_None );
-   return Py_None;
+   Py_RETURN_NONE;
 }
 
 static char crypto_X509_get_issuer_doc[] = "\n\
@@ -142,8 +123,8 @@ crypto_X509_get_issuer( crypto_X509Obj * self, PyObject * args )
     pyname = crypto_X509Name_New( name, 0 );
     if ( pyname != NULL )
     {
-        pyname->parent_cert = ( PyObject * ) self;
         Py_INCREF( self );
+    	pyname->parent_cert = ( PyObject * ) self;
     }
     return ( PyObject * ) pyname;
 }
@@ -196,8 +177,7 @@ crypto_X509_set_issuer( crypto_X509Obj * self, PyObject * args )
         return NULL;
     }
 
-    Py_INCREF( Py_None );
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 static char crypto_X509_get_subject_doc[] = "\n\
@@ -221,8 +201,8 @@ crypto_X509_get_subject( crypto_X509Obj * self, PyObject * args )
     pyname = crypto_X509Name_New( name, 0 );
     if ( pyname != NULL )
     {
-        pyname->parent_cert = ( PyObject * ) self;
         Py_INCREF( self );
+        pyname->parent_cert = ( PyObject * ) self;
     }
     return ( PyObject * ) pyname;
 }
@@ -251,8 +231,7 @@ crypto_X509_set_subject( crypto_X509Obj * self, PyObject * args )
         return NULL;
     }
 
-    Py_INCREF( Py_None );
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 static char crypto_X509_get_pubkey_doc[] = "\n\
@@ -305,8 +284,7 @@ crypto_X509_set_pubkey( crypto_X509Obj * self, PyObject * args )
         return NULL;
     }
 
-    Py_INCREF( Py_None );
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 static char crypto_X509_gmtime_adj_notBefore_doc[] = "\n\
@@ -328,8 +306,7 @@ crypto_X509_gmtime_adj_notBefore( crypto_X509Obj * self, PyObject * args )
 
     X509_gmtime_adj( X509_get_notBefore( self->x509 ), i );
 
-    Py_INCREF( Py_None );
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 static char crypto_X509_gmtime_adj_notAfter_doc[] = "\n\
@@ -351,8 +328,7 @@ crypto_X509_gmtime_adj_notAfter( crypto_X509Obj * self, PyObject * args )
 
     X509_gmtime_adj( X509_get_notAfter( self->x509 ), i );
 
-    Py_INCREF( Py_None );
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 static char crypto_X509_sign_doc[] = "\n\
@@ -388,8 +364,7 @@ crypto_X509_sign( crypto_X509Obj * self, PyObject * args )
         return NULL;
     }
 
-    Py_INCREF( Py_None );
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 static char crypto_X509_verify_pkey_is_issuer_doc[] = "\n\
@@ -434,90 +409,9 @@ crypto_X509_has_expired( crypto_X509Obj * self, PyObject * args )
     tnow = time( NULL );
     if ( ASN1_UTCTIME_cmp_time_t( X509_get_notAfter( self->x509 ), tnow ) <
          0 )
-        return PyInt_FromLong( 1L );
+    	Py_RETURN_TRUE;
     else
-        return PyInt_FromLong( 0L );
-}
-
-unsigned short
-convertASN1_TIMETotm( const ASN1_TIME * asn1Time, struct tm *time_tm )
-{
-    unsigned char *asn1String;
-    int len;
-    char zone;
-
-    asn1String = ASN1_STRING_data( asn1Time );
-    len = strlen( ( char * ) asn1String );
-    /* dont understand */
-    if ( ( len != 13 ) && ( len != 15 ) )
-    {
-        return 0;
-    }
-
-    if ( len == 13 )
-    {
-        len = sscanf( ( char * ) asn1String, "%02d%02d%02d%02d%02d%02d%c",
-                      &( time_tm->tm_year ),
-                      &( time_tm->tm_mon ),
-                      &( time_tm->tm_mday ),
-                      &( time_tm->tm_hour ),
-                      &( time_tm->tm_min ), &( time_tm->tm_sec ), &zone );
-        //HACK: We don't expect this code to run past 2100s or receive certs pre-2000
-        time_tm->tm_year += 2000;
-        /* dont understand */
-        if ( ( len != 7 ) || ( zone != 'Z' ) )
-        {
-            return 0;
-        }
-    }
-
-    if ( len == 15 )
-    {
-        len = sscanf( ( char * ) asn1String, "20%02d%02d%02d%02d%02d%02d%c",
-                      &( time_tm->tm_year ),
-                      &( time_tm->tm_mon ),
-                      &( time_tm->tm_mday ),
-                      &( time_tm->tm_hour ),
-                      &( time_tm->tm_min ), &( time_tm->tm_sec ), &zone );
-        /* dont understand */
-        if ( ( len != 7 ) || ( zone != 'Z' ) )
-        {
-            return 0;
-        }
-    }
-#ifdef _BSD_SOURCE
-    time_tm->tm_zone = zone;
-#endif
-
-    return 1;
-}
-
-static PyObject *
-convertASN1_TIMEToDateTime( const ASN1_TIME * asn1Time )
-{
-    PyObject *datetime;
-    struct tm time_tm;
-
-    if ( !convertASN1_TIMETotm( asn1Time, &time_tm ) )
-    {
-        Py_INCREF( Py_None );
-        return Py_None;
-    }
-
-    PyDateTime_IMPORT;
-    datetime = PyDateTime_FromDateAndTime( time_tm.tm_year,
-                                           time_tm.tm_mon,
-                                           time_tm.tm_mday,
-                                           time_tm.tm_hour,
-                                           time_tm.tm_min,
-                                           time_tm.tm_sec, 0 );
-    /* dont understand */
-    if ( !datetime )
-    {
-        Py_INCREF( Py_None );
-        return Py_None;
-    }
-    return datetime;
+    	Py_RETURN_FALSE;
 }
 
 static char crypto_X509_get_not_after_doc[] = "\n\
@@ -593,7 +487,7 @@ crypto_X509_digest( crypto_X509Obj * self, PyObject * args )
     unsigned char fp[EVP_MAX_MD_SIZE];
     char *tmp;
     char *digest_name;
-    int len, i;
+    unsigned int len, i;
     PyObject *ret;
     const EVP_MD *digest;
 
@@ -699,13 +593,17 @@ crypto_X509_add_extensions( crypto_X509Obj * self, PyObject * args )
 
     nr_of_extensions = PySequence_Fast_GET_SIZE( seq );
 
+    /*
+     * ADRIFIX: Removed Py_DECREF( extList ) because it's a param
+     * 			seq comes from PySequence_Fast and it's a New ref
+     */
+
     for ( i = 0; i < nr_of_extensions; i++ )
     {
         ext =
             ( crypto_X509ExtensionObj * ) PySequence_Fast_GET_ITEM( seq, i );
         if ( !crypto_X509Extension_Check( ext ) )
         {
-            Py_DECREF( extList );
             Py_DECREF( seq );
             PyErr_SetString( PyExc_ValueError,
                              "One of the elements is not an X509Extension" );
@@ -713,17 +611,14 @@ crypto_X509_add_extensions( crypto_X509Obj * self, PyObject * args )
         }
         if ( !X509_add_ext( self->x509, ext->x509_extension, -1 ) )
         {
-            Py_DECREF( extList );
             Py_DECREF( seq );
             exception_from_error_queue(  );
             return NULL;
         }
     }
 
-    Py_DECREF( extList );
     Py_DECREF( seq );
-    Py_INCREF( Py_None );
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 /*
@@ -796,7 +691,7 @@ static void
 crypto_X509_dealloc( crypto_X509Obj * self )
 {
     /* Sometimes we don't have to dealloc the "real" X509 pointer ourselves */
-    if ( self->dealloc )
+    if ( self->dealloc && self->x509 )
     {
         X509_free( self->x509 );
         self->x509 = NULL;
