@@ -1,21 +1,7 @@
-
-/*
- * x509name.c
- *
- * Copyright (C) AB Strakt 2001, All rights reserved
- *
- * X.509 Name handling, mostly thin wrapping.
- * See the file RATIONALE for a short explanation of why this module was written.
- *
- * Reviewed 2001-07-23
- */
 #include <Python.h>
 #define crypto_MODULE
 #include "crypto.h"
 
-static char *CVSid =
-    "@(#) $Id: x509name.c,v 1.11 2008/07/08 10:54:54 acasajus Exp $";
-
 /*
  * Return a name string given a X509_NAME object and a name identifier. Used
  * by the getattr function.
@@ -166,10 +152,10 @@ crypto_X509Name_get_entry( crypto_X509NameObj * self, PyObject * args )
     l = ASN1_STRING_length( fval );
 
     tuple = PyTuple_New( 3 );
-    PyTuple_SetItem( tuple, 0, PyString_FromString( OBJ_nid2sn( nid ) ) );
-    PyTuple_SetItem( tuple, 1,
-                     PyString_FromStringAndSize( ( char * ) str, l ) );
-    PyTuple_SetItem( tuple, 2, PyInt_FromLong( ent->value->type ) );
+    PyTuple_SET_ITEM( tuple, 0, PyString_FromString( OBJ_nid2sn( nid ) ) );
+    PyTuple_SET_ITEM( tuple, 1,
+                      PyString_FromStringAndSize( ( char * ) str, l ) );
+    PyTuple_SET_ITEM( tuple, 2, PyInt_FromLong( ent->value->type ) );
 
     return tuple;
 }
@@ -241,8 +227,7 @@ crypto_X509Name_insert_entry( crypto_X509NameObj * self, PyObject * args )
     }
 
     set_name_by_nid( self->x509_name, nid, type, value, pos, 0 );
-    Py_INCREF( Py_None );
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 static char crypto_X509Name_clone_doc[] = "\n\
@@ -365,11 +350,11 @@ crypto_X509Name_get_components( crypto_X509NameObj * self, PyObject * args )
         /* printf("fname is %s len=%d str=%s\n", OBJ_nid2sn(nid), l, str); */
 
         tuple = PyTuple_New( 2 );
-        PyTuple_SetItem( tuple, 0, PyString_FromString( OBJ_nid2sn( nid ) ) );
-        PyTuple_SetItem( tuple, 1,
-                         PyString_FromStringAndSize( ( char * ) str, l ) );
+        PyTuple_SET_ITEM( tuple, 0, PyString_FromString( OBJ_nid2sn( nid ) ) );
+        PyTuple_SET_ITEM( tuple, 1,
+                          PyString_FromStringAndSize( ( char * ) str, l ) );
 
-        PyList_SetItem( list, i, tuple );
+        PyList_SET_ITEM( list, i, tuple );
     }
 
     return list;
@@ -461,8 +446,7 @@ crypto_X509Name_getattr( crypto_X509NameObj * self, char *name )
         return NULL;
     else if ( len == 0 )
     {
-        Py_INCREF( Py_None );
-        return Py_None;
+    	Py_RETURN_NONE;
     }
     else
     {
@@ -590,7 +574,8 @@ crypto_X509Name_traverse( crypto_X509NameObj * self, visitproc visit,
 static int
 crypto_X509Name_clear( crypto_X509NameObj * self )
 {
-    Py_CLEAR( self->parent_cert );
+	Py_CLEAR( self->parent_cert );
+
     return 0;
 }
 
@@ -605,11 +590,12 @@ crypto_X509Name_dealloc( crypto_X509NameObj * self )
 {
     PyObject_GC_UnTrack( self );
 
-    crypto_X509Name_clear( self );
     /* Sometimes we don't have to dealloc this */
     if ( self->dealloc )
         X509_NAME_free( self->x509_name );
 
+    crypto_X509Name_clear( self );
+
     PyObject_GC_Del( self );
 }
 
