@@ -1,14 +1,4 @@
 
-/*
- * connection.c
- *
- * Copyright (C) AB Strakt 2001, All rights reserved
- *
- * SSL Connection objects and methods.
- * See the file RATIONALE for a short explanation of why this module was written.
- *
- * Reviewed 2001-07-23
- */
 #include <Python.h>
 #define SSL_MODULE
 #include <openssl/err.h>
@@ -24,8 +14,6 @@
 #  include <winsock.h>
 #endif
 
-static char *CVSid =
-    "@(#) $Id: connection.c,v 1.7 2008/07/08 10:54:55 acasajus Exp $";
 //static int handshaked = 0;
 
 /**
@@ -321,24 +309,18 @@ ssl_Connection_send( ssl_ConnectionObj * self, PyObject * args )
     if ( !PyArg_ParseTuple( args, "s#|i:send", &buf, &len, &flags ) )
         return NULL;
 
-    MY_BEGIN_ALLOW_THREADS( self->tstate )
-        ret = SSL_write( self->ssl, buf, len );
-    MY_END_ALLOW_THREADS( self->tstate ) if ( PyErr_Occurred(  ) )
-    {
-        flush_error_queue(  );
-        return NULL;
-    }
+    OBJ_BEGIN_THREADS( self );
+    ret = SSL_write( self->ssl, buf, len );
+    OBJ_END_THREADS( self );
 
     err = SSL_get_error( self->ssl, ret );
-    if ( err == SSL_ERROR_NONE )
+    if ( err != SSL_ERROR_NONE )
     {
-        return PyInt_FromLong( ( long ) ret );
-    }
-    else
-    {
-        handle_ssl_errors( self->ssl, err, ret );
-        return NULL;
+    	handle_ssl_errors( self->ssl, err, ret );
+    	return NULL;
     }
+
+    return PyInt_FromLong( ( long ) ret );
 }
 
 static char ssl_Connection_sendall_doc[] = "\n\
@@ -358,39 +340,28 @@ ssl_Connection_sendall( ssl_ConnectionObj * self, PyObject * args )
 {
     char *buf;
     int len, ret, err, flags;
-    PyObject *pyret = Py_None;
 
     if ( !PyArg_ParseTuple( args, "s#|i:sendall", &buf, &len, &flags ) )
         return NULL;
 
     do
     {
-        MY_BEGIN_ALLOW_THREADS( self->tstate )
-            ret = SSL_write( self->ssl, buf, len );
-        MY_END_ALLOW_THREADS( self->tstate ) if ( PyErr_Occurred(  ) )
-        {
-            flush_error_queue(  );
-            pyret = NULL;
-            break;
-        }
+        OBJ_BEGIN_THREADS( self );
+        ret = SSL_write( self->ssl, buf, len );
+        OBJ_END_THREADS( self );
+
         err = SSL_get_error( self->ssl, ret );
-        if ( err == SSL_ERROR_NONE )
+        if ( err != SSL_ERROR_NONE )
         {
-            buf += ret;
-            len -= ret;
-        }
-        else if ( err == SSL_ERROR_SSL || err == SSL_ERROR_SYSCALL ||
-                  err == SSL_ERROR_ZERO_RETURN )
-        {
-            handle_ssl_errors( self->ssl, err, ret );
-            pyret = NULL;
-            break;
+        	handle_ssl_errors( self->ssl, err, ret );
+        	return NULL;
         }
+		buf += ret;
+		len -= ret;
     }
     while ( len > 0 );
 
-    Py_XINCREF( pyret );
-    return pyret;
+    Py_RETURN_NONE;
 }
 
 static char ssl_Connection_recv_doc[] = "\n\
@@ -415,34 +386,67 @@ ssl_Connection_recv( ssl_ConnectionObj * self, PyObject * args )
     if ( !PyArg_ParseTuple( args, "i|i:recv", &bufsiz, &flags ) )
         return NULL;
 
+	if( bufsiz <= 0 )
+	{
+		//Raise exception
+		return NULL;
+	}
+
+    cbuf = OPENSSL_malloc( sizeof( char ) * bufsiz );
+    if( !cbuf )
+    	return NULL;
+
+    OBJ_BEGIN_THREADS( self );
+    ret = SSL_read( self->ssl, cbuf, bufsiz );
+    OBJ_END_THREADS( self );
+
+    err = SSL_get_error( self->ssl, ret );
+    if ( err != SSL_ERROR_NONE )
+    {
+    	handle_ssl_errors( self->ssl, err, ret );
+    	OPENSSL_free( cbuf );
+    	return NULL;
+    }
+
+    buf = PyString_FromStringAndSize( cbuf, ret );
+
+    OPENSSL_free( cbuf );
+
+    if( !buf )
+    	return NULL;
+
+    return buf;
+}
+
+    /*
+
     buf = PyString_FromStringAndSize( NULL, bufsiz );
     if ( buf == NULL )
         return NULL;
 
     cbuf = PyString_AsString( buf );
-    MY_BEGIN_ALLOW_THREADS( self->tstate )
-        ret = SSL_read( self->ssl, cbuf, bufsiz );
-    MY_END_ALLOW_THREADS( self->tstate ) if ( PyErr_Occurred(  ) )
-    {
-        Py_DECREF( buf );
-        flush_error_queue(  );
-        return NULL;
-    }
+
+    OBJ_BEGIN_THREADS( self );
+    ret = SSL_read( self->ssl, cbuf, bufsiz );
+    OBJ_END_THREADS( self );
 
     err = SSL_get_error( self->ssl, ret );
-    if ( err == SSL_ERROR_NONE )
+    if ( err != SSL_ERROR_NONE )
     {
-        if ( ret != bufsiz && _PyString_Resize( &buf, ret ) < 0 )
-            return NULL;
-        return buf;
+    	Py_DECREF( buf );
+    	handle_ssl_errors( self->ssl, err, ret );
+    	return NULL;
     }
-    else
+
+
+    if ( ret != bufsiz && _PyString_Resize( &buf, ret ) < 0 )
     {
-        handle_ssl_errors( self->ssl, err, ret );
-        Py_DECREF( buf );
+		Py_DECREF( buf );
         return NULL;
     }
+    return buf;
 }
+*/
 
 static char ssl_Connection_renegotiate_doc[] = "\n\
 Renegotiate the session\n\
@@ -459,9 +463,9 @@ ssl_Connection_renegotiate( ssl_ConnectionObj * self, PyObject * args )
     if ( !PyArg_ParseTuple( args, ":renegotiate" ) )
         return NULL;
 
-    MY_BEGIN_ALLOW_THREADS( self->tstate );
+    OBJ_BEGIN_THREADS( self );
     ret = SSL_renegotiate( self->ssl );
-    MY_END_ALLOW_THREADS( self->tstate );
+    OBJ_END_THREADS( self );
 
     if ( PyErr_Occurred(  ) )
     {
@@ -547,26 +551,25 @@ ssl_Connection_do_handshake( ssl_ConnectionObj * self, PyObject * args )
     if ( !PyArg_ParseTuple( args, ":do_handshake" ) )
         return NULL;
 
-    MY_BEGIN_ALLOW_THREADS( self->tstate );
+    OBJ_BEGIN_THREADS( self );
     ret = SSL_do_handshake( self->ssl );
-    MY_END_ALLOW_THREADS( self->tstate );
+    OBJ_END_THREADS( self );
 
-    if ( PyErr_Occurred(  ) )
+    if ( PyErr_Occurred() )
     {
-        flush_error_queue(  );
+        flush_error_queue();
         return NULL;
     }
 
     err = SSL_get_error( self->ssl, ret );
     if ( err == SSL_ERROR_NONE )
     {
-        Py_INCREF( Py_None );
-        return Py_None;
+    	Py_RETURN_NONE;
     }
     else
     {
         helper_treatHandshakeError( self, err, ret );
-        flush_error_queue(  );
+        flush_error_queue();
         return NULL;
     }
 }
@@ -624,8 +627,7 @@ ssl_Connection_set_accept_state( ssl_ConnectionObj * self, PyObject * args )
 
     SSL_set_accept_state( self->ssl );
 
-    Py_INCREF( Py_None );
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 static char ssl_Connection_set_connect_state_doc[] = "\n\
@@ -644,8 +646,7 @@ ssl_Connection_set_connect_state( ssl_ConnectionObj * self, PyObject * args )
 
     SSL_set_connect_state( self->ssl );
 
-    Py_INCREF( Py_None );
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 static char ssl_Connection_connect_doc[] = "\n\
@@ -768,8 +769,11 @@ ssl_Connection_shutdown( ssl_ConnectionObj * self, PyObject * args )
     if ( !PyArg_ParseTuple( args, ":shutdown" ) )
         return NULL;
 
-    MY_BEGIN_ALLOW_THREADS( self->tstate ) ret = SSL_shutdown( self->ssl );
-    MY_END_ALLOW_THREADS( self->tstate ) if ( PyErr_Occurred(  ) )
+    OBJ_BEGIN_THREADS( self );
+    ret = SSL_shutdown( self->ssl );
+    OBJ_END_THREADS( self );
+
+    if ( PyErr_Occurred(  ) )
     {
         flush_error_queue(  );
         return NULL;
@@ -781,15 +785,9 @@ ssl_Connection_shutdown( ssl_ConnectionObj * self, PyObject * args )
         return NULL;
     }
     else if ( ret > 0 )
-    {
-        Py_INCREF( Py_True );
-        return Py_True;
-    }
+        Py_RETURN_TRUE;
     else
-    {
-        Py_INCREF( Py_False );
-        return Py_False;
-    }
+        Py_RETURN_FALSE;
 }
 
 static char ssl_Connection_get_cipher_list_doc[] = "\n\
@@ -867,17 +865,17 @@ Returns:   None\n\
 static PyObject *
 ssl_Connection_set_app_data( ssl_ConnectionObj * self, PyObject * args )
 {
-    PyObject *data;
+    PyObject *data,*old;
 
     if ( !PyArg_ParseTuple( args, "O:set_app_data", &data ) )
         return NULL;
 
-    Py_DECREF( self->app_data );
+    old = self->app_data;
     Py_INCREF( data );
     self->app_data = data;
+    Py_DECREF( old );
 
-    Py_INCREF( Py_None );
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 static char ssl_Connection_state_string_doc[] = "\n\
@@ -940,8 +938,7 @@ ssl_Connection_get_peer_certificate( ssl_ConnectionObj *
     }
     else
     {
-        Py_INCREF( Py_None );
-        return Py_None;
+    	Py_RETURN_NONE;
     }
 }
 
@@ -992,8 +989,7 @@ ssl_Connection_get_peer_certificate_chain( ssl_ConnectionObj *
     }
     else
     {
-        Py_INCREF( Py_None );
-        return Py_None;
+    	Py_RETURN_NONE;
     }
 }
 
@@ -1169,8 +1165,7 @@ ssl_Connection_set_shutdown( ssl_ConnectionObj * self, PyObject * args )
 
     SSL_set_shutdown( self->ssl, shutdownMode );
 
-    Py_INCREF( Py_None );
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 static char ssl_Connection_peek_doc[] = "\n\
@@ -1200,9 +1195,12 @@ ssl_Connection_peek( ssl_ConnectionObj * self, PyObject * args )
         return NULL;
 
     cbuf = PyString_AsString( buf );
-    MY_BEGIN_ALLOW_THREADS( self->tstate )
-        ret = SSL_peek( self->ssl, cbuf, bufsiz );
-    MY_END_ALLOW_THREADS( self->tstate ) if ( PyErr_Occurred(  ) )
+
+    OBJ_BEGIN_THREADS( self );
+    ret = SSL_peek( self->ssl, cbuf, bufsiz );
+    OBJ_END_THREADS( self );
+
+    if ( PyErr_Occurred(  ) )
     {
         Py_DECREF( buf );
         flush_error_queue(  );
@@ -1213,7 +1211,7 @@ ssl_Connection_peek( ssl_ConnectionObj * self, PyObject * args )
     if ( err == SSL_ERROR_NONE )
     {
         if ( ret != bufsiz && _PyString_Resize( &buf, ret ) < 0 )
-            return NULL;
+        return NULL;
         return buf;
     }
     else
@@ -1317,7 +1315,10 @@ ssl_Connection_New( ssl_ContextObj * ctx, PyObject * sock )
     fd = PyObject_AsFileDescriptor( self->socket );
     if ( fd < 0 )
     {
-        Py_DECREF( self );
+        Py_CLEAR( self->context );
+        Py_CLEAR( self->socket );
+        Py_CLEAR( self->app_data );
+        PyObject_GC_Del( self );
         return NULL;
     }
 
@@ -1400,9 +1401,12 @@ static void
 ssl_Connection_dealloc( ssl_ConnectionObj * self )
 {
     PyObject_GC_UnTrack( self );
-    ssl_Connection_clear( self );
+
     if ( self->ssl != NULL )
         SSL_free( self->ssl );
+
+    ssl_Connection_clear( self );
+
     PyObject_GC_Del( self );
 }
 
